@@ -15,7 +15,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #ifndef DPKG_PKG_SHOW_H
@@ -34,8 +34,8 @@ DPKG_BEGIN_DECLS
 
 int pkg_sorter_by_nonambig_name_arch(const void *a, const void *b);
 
-const char *pkg_summary(const struct pkginfo *pkg, const struct pkgbin *pkgbin,
-                        int *len_ret);
+const char *pkgbin_synopsis(const struct pkginfo *pkg,
+                            const struct pkgbin *pkgbin, int *len_ret);
 int pkg_abbrev_want(const struct pkginfo *pkg);
 int pkg_abbrev_status(const struct pkginfo *pkg);
 int pkg_abbrev_eflag(const struct pkginfo *pkg);
