@@ -3,7 +3,7 @@
  * tarfn.h - tar archive extraction functions
  *
  * Copyright © 1995 Bruce Perens
- * Copyright © 2009-2014 Guillem Jover <guillem@debian.org>
+ * Copyright © 2009-2014, 2017 Guillem Jover <guillem@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,6 +24,9 @@
 
 #include <sys/types.h>
 
+#include <stdint.h>
+
+#include <dpkg/error.h>
 #include <dpkg/file.h>
 
 /**
@@ -51,12 +54,21 @@ enum tar_filetype {
 	TAR_FILETYPE_BLOCKDEV = '4',
 	TAR_FILETYPE_DIR = '5',
 	TAR_FILETYPE_FIFO = '6',
+	TAR_FILETYPE_CONTIG = '7',
 	TAR_FILETYPE_GNU_LONGLINK = 'K',
 	TAR_FILETYPE_GNU_LONGNAME = 'L',
+	TAR_FILETYPE_GNU_VOLUME = 'V',
+	TAR_FILETYPE_GNU_MULTIVOL = 'M',
+	TAR_FILETYPE_GNU_DUMPDIR = 'D',
+	TAR_FILETYPE_GNU_SPARSE = 'S',
+	TAR_FILETYPE_PAX_GLOBAL = 'g',
+	TAR_FILETYPE_PAX_EXTENDED = 'x',
+	TAR_FILETYPE_SOLARIS_EXTENDED = 'X',
+	TAR_FILETYPE_SOLARIS_ACL = 'A',
 };
 
 struct tar_entry {
-	/** Tar archive format. */
+	/** Tar entry format. */
 	enum tar_format format;
 	/** File type. */
 	enum tar_filetype type;
@@ -74,8 +86,10 @@ struct tar_entry {
 	struct file_stat stat;
 };
 
-typedef int tar_read_func(void *ctx, char *buffer, int length);
-typedef int tar_make_func(void *ctx, struct tar_entry *h);
+struct tar_archive;
+
+typedef int tar_read_func(struct tar_archive *tar, char *buffer, int length);
+typedef int tar_make_func(struct tar_archive *tar, struct tar_entry *h);
 
 struct tar_operations {
 	tar_read_func *read;
@@ -87,10 +101,28 @@ struct tar_operations {
 	tar_make_func *mknod;
 };
 
+struct tar_archive {
+	/* Global tar archive error. */
+	struct dpkg_error err;
+
+	/** Tar archive format. */
+	enum tar_format format;
+
+	/* Operation functions and context. */
+	const struct tar_operations *ops;
+	void *ctx;
+};
+
+uintmax_t
+tar_atoul(const char *s, size_t size, uintmax_t max);
+intmax_t
+tar_atosl(const char *s, size_t size, intmax_t min, intmax_t max);
+
 void
 tar_entry_update_from_system(struct tar_entry *te);
 
-int tar_extractor(void *ctx, const struct tar_operations *ops);
+int
+tar_extractor(struct tar_archive *tar);
 
 /** @} */
 
