@@ -192,100 +192,12 @@ static const char printforhelp[] = N_(
 int f_pending=0, f_recursive=0, f_alsoselect=1, f_skipsame=0, f_noact=0;
 int f_autodeconf=0, f_nodebsig=0;
 int f_triggers = 0;
-int fc_downgrade=1, fc_configureany=0, fc_hold=0, fc_removereinstreq=0, fc_overwrite=0;
-int fc_removeessential=0, fc_conflicts=0, fc_depends=0, fc_dependsversion=0;
-int fc_breaks=0, fc_badpath=0, fc_overwritediverted=0, fc_architecture=0;
-int fc_nonroot=0, fc_overwritedir=0, fc_conff_new=0, fc_conff_miss=0;
-int fc_conff_old=0, fc_conff_def=0;
-int fc_conff_ask = 0;
-int fc_unsafe_io = 0;
-int fc_badverify = 0;
-int fc_badversion = 0;
-int fc_script_chrootless = 0;
 
 int errabort = 50;
 static const char *admindir = ADMINDIR;
 const char *instdir= "";
 struct pkg_list *ignoredependss = NULL;
 
-static const char *
-forcetype_str(char type)
-{
-  switch (type) {
-  case '\0':
-  case ' ':
-    return "   ";
-  case '*':
-    return "[*]";
-  case '!':
-    return "[!]";
-  default:
-    internerr("unknown force type '%c'", type);
-  }
-}
-
-static const struct forceinfo {
-  const char *name;
-  int *opt;
-  char type;
-  const char *desc;
-} forceinfos[]= {
-  { "all",                 NULL,
-    '!', N_("Set all force options")},
-  { "downgrade",           &fc_downgrade,
-    '*', N_("Replace a package with a lower version") },
-  { "configure-any",       &fc_configureany,
-    ' ', N_("Configure any package which may help this one") },
-  { "hold",                &fc_hold,
-    ' ', N_("Process incidental packages even when on hold") },
-  { "not-root",            &fc_nonroot,
-    ' ', N_("Try to (de)install things even when not root") },
-  { "bad-path",            &fc_badpath,
-    ' ', N_("PATH is missing important programs, problems likely") },
-  { "bad-verify",          &fc_badverify,
-    ' ', N_("Install a package even if it fails authenticity check") },
-  { "bad-version",         &fc_badversion,
-    ' ', N_("Process even packages with wrong versions") },
-  { "overwrite",           &fc_overwrite,
-    ' ', N_("Overwrite a file from one package with another") },
-  { "overwrite-diverted",  &fc_overwritediverted,
-    ' ', N_("Overwrite a diverted file with an undiverted version") },
-  { "overwrite-dir",       &fc_overwritedir,
-    '!', N_("Overwrite one package's directory with another's file") },
-  { "unsafe-io",           &fc_unsafe_io,
-    '!', N_("Do not perform safe I/O operations when unpacking") },
-  { "script-chrootless",   &fc_script_chrootless,
-    '!', N_("Do not chroot into maintainer script environment") },
-  { "confnew",             &fc_conff_new,
-    '!', N_("Always use the new config files, don't prompt") },
-  { "confold",             &fc_conff_old,
-    '!', N_("Always use the old config files, don't prompt") },
-  { "confdef",             &fc_conff_def,
-    '!', N_("Use the default option for new config files if one\n"
-            "is available, don't prompt. If no default can be found,\n"
-            "you will be prompted unless one of the confold or\n"
-            "confnew options is also given") },
-  { "confmiss",            &fc_conff_miss,
-    '!', N_("Always install missing config files") },
-  { "confask",             &fc_conff_ask,
-    '!', N_("Offer to replace config files with no new versions") },
-  { "architecture",        &fc_architecture,
-    '!', N_("Process even packages with wrong or no architecture") },
-  { "breaks",              &fc_breaks,
-    '!', N_("Install even if it would break another package") },
-  { "conflicts",           &fc_conflicts,
-    '!', N_("Allow installation of conflicting packages") },
-  { "depends",             &fc_depends,
-    '!', N_("Turn all dependency problems into warnings") },
-  { "depends-version",     &fc_dependsversion,
-    '!', N_("Turn dependency version problems into warnings") },
-  { "remove-reinstreq",    &fc_removereinstreq,
-    '!', N_("Remove packages which require installation") },
-  { "remove-essential",    &fc_removeessential,
-    '!', N_("Remove an essential package") },
-  { NULL }
-};
-
 #define DBG_DEF(n, d) \
   { .flag = dbg_##n, .name = #n, .desc = d }
 
@@ -580,7 +492,7 @@ arch_remove(const char *const *argv)
     if (pkg->status < PKG_STAT_HALFINSTALLED)
       continue;
     if (pkg->installed.arch == arch) {
-      if (fc_architecture)
+      if (in_force(FORCE_ARCHITECTURE))
         warning(_("removing architecture '%s' currently in use by database"),
                 arch->name);
       else
@@ -599,78 +511,6 @@ arch_remove(const char *const *argv)
   return 0;
 }
 
-static inline void
-print_forceinfo_line(int type, const char *name, const char *desc)
-{
-  printf("  %s %-18s %s\n", forcetype_str(type), name, desc);
-}
-
-static void
-print_forceinfo(const struct forceinfo *fi)
-{
-  char *desc, *line;
-
-  desc = m_strdup(gettext(fi->desc));
-
-  line = strtok(desc, "\n");
-  print_forceinfo_line(fi->type, fi->name, line);
-  while ((line = strtok(NULL, "\n")))
-    print_forceinfo_line(' ', "", line);
-
-  free(desc);
-}
-
-static void
-set_force(const struct cmdinfo *cip, const char *value)
-{
-  const char *comma;
-  size_t l;
-  const struct forceinfo *fip;
-
-  if (strcmp(value, "help") == 0) {
-    printf(_(
-"%s forcing options - control behaviour when problems found:\n"
-"  warn but continue:  --force-<thing>,<thing>,...\n"
-"  stop with error:    --refuse-<thing>,<thing>,... | --no-force-<thing>,...\n"
-" Forcing things:\n"), DPKG);
-
-    for (fip = forceinfos; fip->name; fip++)
-      print_forceinfo(fip);
-
-    printf(_(
-"\n"
-"WARNING - use of options marked [!] can seriously damage your installation.\n"
-"Forcing options marked [*] are enabled by default.\n"));
-    m_output(stdout, _("<standard output>"));
-    exit(0);
-  }
-
-  for (;;) {
-    comma = strchrnul(value, ',');
-    l = (size_t)(comma - value);
-    for (fip=forceinfos; fip->name; fip++)
-      if (strncmp(fip->name, value, l) == 0 && strlen(fip->name) == l)
-        break;
-
-    if (!fip->name) {
-      badusage(_("unknown force/refuse option '%.*s'"),
-               (int)min(l, 250), value);
-    } else if (strcmp(fip->name, "all") == 0) {
-      for (fip = forceinfos; fip->name; fip++)
-        if (fip->opt)
-          *fip->opt = cip->arg_int;
-    } else if (fip->opt) {
-      *fip->opt = cip->arg_int;
-    } else {
-      warning(_("obsolete force/refuse option '%s'"), fip->name);
-    }
-
-    if (*comma == '\0')
-      break;
-    value= ++comma;
-  }
-}
-
 int execbackend(const char *const *argv) DPKG_ATTR_NORET;
 int commandfd(const char *const *argv);
 
@@ -739,7 +579,7 @@ static const struct cmdinfo cmdinfos[]= {
   { "no-pager",          0,   0, NULL,          NULL,      set_no_pager,  0 },
   { "no-debsig",         0,   0, &f_nodebsig,   NULL,      NULL,    1 },
   /* Alias ('G') for --refuse. */
-  {  NULL,               'G', 0, &fc_downgrade, NULL,      NULL,    0 },
+  {  NULL,               'G', 0, NULL,          NULL,      reset_force_option, FORCE_DOWNGRADE },
   { "selected-only",     'O', 0, &f_alsoselect, NULL,      NULL,    0 },
   { "triggers",           0,  0, &f_triggers,   NULL,      NULL,    1 },
   { "no-triggers",        0,  0, &f_triggers,   NULL,      NULL,   -1 },
@@ -752,9 +592,9 @@ static const struct cmdinfo cmdinfos[]= {
   { "admindir",          0,   1, NULL,          &admindir, NULL,          0 },
   { "instdir",           0,   1, NULL,          NULL,      set_instdir,   0 },
   { "ignore-depends",    0,   1, NULL,          NULL,      set_ignore_depends, 0 },
-  { "force",             0,   2, NULL,          NULL,      set_force,     1 },
-  { "refuse",            0,   2, NULL,          NULL,      set_force,     0 },
-  { "no-force",          0,   2, NULL,          NULL,      set_force,     0 },
+  { "force",             0,   2, NULL,          NULL,      set_force_option,   1 },
+  { "refuse",            0,   2, NULL,          NULL,      set_force_option,   0 },
+  { "no-force",          0,   2, NULL,          NULL,      set_force_option,   0 },
   { "debug",             'D', 1, NULL,          NULL,      set_debug,     0 },
   { "help",              '?', 0, NULL,          NULL,      usage,         0 },
   { "version",           0,   0, NULL,          NULL,      printversion,  0 },
@@ -903,11 +743,12 @@ int main(int argc, const char *const *argv) {
   dpkg_locales_init(PACKAGE);
   dpkg_program_init("dpkg");
   dpkg_options_load(DPKG, cmdinfos);
+  set_force_default(FORCE_ALL);
   dpkg_options_parse(&argv, cmdinfos, printforhelp);
 
   /* When running as root, make sure our primary group is also root, so
    * that files created by maintainer scripts have correct ownership. */
-  if (!fc_nonroot && getuid() == 0)
+  if (!in_force(FORCE_NON_ROOT) && getuid() == 0)
     if (setgid(0) < 0)
       ohshite(_("cannot set primary group ID to root"));
 
@@ -920,6 +761,8 @@ int main(int argc, const char *const *argv) {
     ohshite(_("unable to setenv for subprocesses"));
   if (setenv("DPKG_ROOT", instdir, 1) < 0)
     ohshite(_("unable to setenv for subprocesses"));
+  if (setenv("DPKG_FORCE", get_force_string(), 1) < 0)
+    ohshite(_("unable to setenv for subprocesses"));
 
   if (!f_triggers)
     f_triggers = (cipaction->arg_int == act_triggers && *argv) ? -1 : 1;
